package ec.edu.epn.consumosriautorizacion.Tasks;

import ec.edu.epn.consumosriautorizacion.controller.ConsultasFacturacionController;
import ec.edu.epn.consumosriautorizacion.dto.FacturaDTO;
import ec.edu.epn.consumosriautorizacion.dto.FacturacionOfflineDTO;
import ec.edu.epn.consumosriautorizacion.service.EmailService;
import ec.gob.sri.comprobantes.ws.RecepcionComprobantesOffline;
import ec.gob.sri.comprobantes.ws.RecepcionComprobantesOfflineService;
import ec.gob.sri.comprobantes.ws.RespuestaSolicitud;
import ec.gob.sri.comprobantes.ws.aut.AutorizacionComprobantesOffline;
import ec.gob.sri.comprobantes.ws.aut.AutorizacionComprobantesOfflineService;
import ec.gob.sri.comprobantes.ws.aut.RespuestaComprobante;
import org.springframework.batch.core.StepContribution;
import org.springframework.batch.core.scope.context.ChunkContext;
import org.springframework.batch.core.step.tasklet.Tasklet;
import org.springframework.batch.repeat.RepeatStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.xml.namespace.QName;
import java.net.URL;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;

import static java.nio.charset.StandardCharsets.ISO_8859_1;
import static java.nio.charset.StandardCharsets.UTF_8;


@Component
public class Task_autorizacion implements Tasklet {





    private static String  ambiente= "PRODUCCION";


   /* @Autowired
    ConsultasFacturacionController consultas;*/


    @Override
    public RepeatStatus execute(StepContribution contribution, ChunkContext chunkContext) throws Exception {

        ConsultasFacturacionController consultas= new ConsultasFacturacionController();

          List<FacturaDTO> listFactContingencia= new ArrayList<FacturaDTO>();

          /**OBTENER LA FECHA ACTUAL**/
        Date date = new Date();
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        listFactContingencia= consultas.consumoFactContingencia(dateFormat.format(date));
        /**RECORREMOS LA LISTA DE FACTURAS PARA SU AUTORIZACION*/
        if(listFactContingencia!=null){
            for(FacturaDTO dto: listFactContingencia){

                /**ENVIO PARA RECEPCION DE FACTURAS**/
                FacturacionOfflineDTO offlineRecepcionDTO = new FacturacionOfflineDTO();
                offlineRecepcionDTO= this.recepcion(dto.getXml());

                if(offlineRecepcionDTO.getEstado().equals("RECIBIDA")){

                    this.envioCorreoRecepcion(dto);

                    dto.setEstadoFactura(offlineRecepcionDTO.getEstado());

                    try {
                        consultas.actualizarEstadoSRIFact(dto);
                    }catch(Exception e){
                        return RepeatStatus.FINISHED;
                    }

                    /**ENVIO PARA AUTORIZACIÓN DE FACTURA**/
                    FacturacionOfflineDTO offlineAutorizacionDTO = new FacturacionOfflineDTO();

                    offlineAutorizacionDTO= this.autorizacion(dto.getNumautoriza());

                    if(offlineAutorizacionDTO!=null){

                        if(offlineAutorizacionDTO.getEstado().equals("AUTORIZADO")){
                            dto.setFechaautoriza(offlineAutorizacionDTO.getFechaAutorizacion().toString());
                            dto.setEstadoFactura(offlineAutorizacionDTO.getEstado());


                            try {
                                consultas.actualizarSRIFact(dto);
                            }catch(Exception e){
                                return RepeatStatus.FINISHED;
                            }

                        }else{
                            offlineAutorizacionDTO.setNumeroAutoFac(dto.getIdFactura());
                            consultas.ingresoErrorSRIFact(offlineAutorizacionDTO);
                        }

                    }
                }else{

                    offlineRecepcionDTO.setNumeroAutoFac(dto.getIdFactura());
                    consultas.ingresoErrorSRIFact(offlineRecepcionDTO);

                }

            }
        }
        return  RepeatStatus.FINISHED;

    }




    public FacturacionOfflineDTO recepcion(String xml){

        String val = "";
        FacturacionOfflineDTO offlineDTO = new FacturacionOfflineDTO();

        try {
            RespuestaSolicitud respuesta = new RespuestaSolicitud();

            byte[] bytes = xml.getBytes();

            String wsdl = "https://cel.sri.gob.ec/comprobantes-electronicos-ws/RecepcionComprobantesOffline?wsdl";
            URL url = new URL(wsdl);

            QName qname = new QName("http://ec.gob.sri.ws.recepcion", "RecepcionComprobantesOfflineService");

            RecepcionComprobantesOfflineService service = new RecepcionComprobantesOfflineService(url, qname);
            RecepcionComprobantesOffline port = service.getRecepcionComprobantesOfflinePort();  //getRecepcionComprobantesOfflinePort();

            respuesta = port.validarComprobante(bytes);

            /*PARA EL CASO DONDE EL ESTADO SEA NULL**/
            if (respuesta.getEstado() == null) {
                if (respuesta.getComprobantes() != null) {

                    try {
                        offlineDTO.setMensaje(respuesta.getComprobantes().getComprobante().get(0).getMensajes().getMensaje().get(0).getMensaje());
                    } catch (Exception e) {
                        offlineDTO.setMensaje("NINGUN MENSAJE ENCONTRADO");
                    }

                    try {
                        offlineDTO.setInfoAdicional(respuesta.getComprobantes().getComprobante().get(0).getMensajes().getMensaje().get(0).getInformacionAdicional());
                    } catch (Exception e) {
                        offlineDTO.setInfoAdicional("NO DEFINIDO");
                    }

                    try {
                        offlineDTO.setIdentificador(Integer.valueOf(respuesta.getComprobantes().getComprobante().get(0).getMensajes().getMensaje().get(0).getIdentificador()));
                    } catch (Exception e) {
                        offlineDTO.setIdentificador(0);
                    }

                    try {
                        offlineDTO.setTipo(respuesta.getComprobantes().getComprobante().get(0).getMensajes().getMensaje().get(0).getTipo());
                    } catch (Exception e) {
                        offlineDTO.setTipo("ERROR");
                    }

                    try {
                        offlineDTO.setClaveAcceso(respuesta.getComprobantes().getComprobante().get(0).getClaveAcceso());
                    } catch (Exception e) {
                        offlineDTO.setClaveAcceso("NO DEFINIDO");
                    }

                    offlineDTO.setEstado("DEVUELTA - SN");

                }
            }else{
                if (respuesta.getEstado().equals("DEVUELTA")) {
                    try {
                        offlineDTO.setMensaje(respuesta.getComprobantes().getComprobante().get(0).getMensajes().getMensaje().get(0).getMensaje());
                    } catch (Exception e) {
                        offlineDTO.setMensaje("NINGUN MENSAJE ENCOTRADO");
                    }

                    try {
                        offlineDTO.setInfoAdicional(respuesta.getComprobantes().getComprobante().get(0).getMensajes().getMensaje().get(0).getInformacionAdicional());
                    } catch (Exception e) {
                        offlineDTO.setInfoAdicional("NO DEFINIDO");
                    }

                    try {
                        offlineDTO.setIdentificador(Integer.valueOf(respuesta.getComprobantes().getComprobante().get(0).getMensajes().getMensaje().get(0).getIdentificador()));
                    } catch (Exception e) {
                        offlineDTO.setIdentificador(0);
                    }

                    try {
                        offlineDTO.setTipo(respuesta.getComprobantes().getComprobante().get(0).getMensajes().getMensaje().get(0).getTipo());
                    } catch (Exception e) {
                        offlineDTO.setTipo("ERROR");
                    }

                    try {
                        offlineDTO.setClaveAcceso(respuesta.getComprobantes().getComprobante().get(0).getClaveAcceso());
                    } catch (Exception e) {
                        offlineDTO.setClaveAcceso("NO DEFINIDO");
                    }

                    offlineDTO.setEstado("DEVUELTA");
                }

                if (respuesta.getEstado().equals("RECIBIDA")) {
                    offlineDTO.setEstado("RECIBIDA");
                }

            }



            System.out.println("ESTADO FACTURA  " + respuesta.getEstado());

            return offlineDTO;

        } catch (Exception e) {

            return offlineDTO;
        }
    }


    public FacturacionOfflineDTO autorizacion(String numAutorizacion){

        FacturacionOfflineDTO offlineDTO = new FacturacionOfflineDTO();
        try {

            String wsdl = "https://cel.sri.gob.ec/comprobantes-electronicos-ws/AutorizacionComprobantesOffline?wsdl";
            URL url = new URL(wsdl);
            QName qname = new QName("http://ec.gob.sri.ws.autorizacion", "AutorizacionComprobantesOfflineService");
            AutorizacionComprobantesOfflineService services = new AutorizacionComprobantesOfflineService(new URL(wsdl), qname);
            AutorizacionComprobantesOffline port = services.getAutorizacionComprobantesOfflinePort();


            RespuestaComprobante respuesta = port.autorizacionComprobante(numAutorizacion);

            if(respuesta.getAutorizaciones()!=null){

                if(respuesta.getAutorizaciones().getAutorizacion().get(0).getEstado().equals("AUTORIZADO")){
                    offlineDTO.setAmbiente("PRODUCCION");
                    offlineDTO.setEstado("AUTORIZADO");

                    try {
                        respuesta.getAutorizaciones().getAutorizacion().get(0).getFechaAutorizacion().toGregorianCalendar().setTimeZone(TimeZone.getTimeZone("ECT"));
                        offlineDTO.setFechaAutorizacion(Timestamp.valueOf(respuesta.getAutorizaciones().getAutorizacion().get(0).getFechaAutorizacion().toGregorianCalendar().getTimeZone().toString()));
                    } catch (Exception e) {

                        Calendar cal = Calendar.getInstance();
                        Date fechaHoy= cal.getTime();
                        String DATE_FORMAT = "yyyy-MM-dd hh:mm:ss";
                        TimeZone timeZone = TimeZone.getTimeZone("ECT");
                        SimpleDateFormat formatterWithTimeZone = new SimpleDateFormat(DATE_FORMAT);
                        formatterWithTimeZone.setTimeZone(timeZone);
                        String sDate = formatterWithTimeZone.format(fechaHoy);
                        SimpleDateFormat formatter = new SimpleDateFormat(DATE_FORMAT);
                        Date dateWithTimeZone = formatter.parse(sDate);




                        offlineDTO.setFechaAutorizacion(new Timestamp(dateWithTimeZone.getTime()));
                    }

                }

                if(!respuesta.getAutorizaciones().getAutorizacion().get(0).getEstado().equals("AUTORIZADO")){

                    offlineDTO.setAmbiente("PRODUCCION");
                    offlineDTO.setEstado("NO AUTORIZADO -ND");
                    try {
                        offlineDTO.setMensaje(respuesta.getAutorizaciones().getAutorizacion().get(0).getMensajes().getMensaje().get(0).getMensaje());
                    } catch (Exception e) {
                        offlineDTO.setMensaje("NO DEFIIDO");
                    }


                    try {
                        offlineDTO.setIdentificador(Integer.valueOf(respuesta.getAutorizaciones().getAutorizacion().get(0).getMensajes().getMensaje().get(0).getIdentificador()));
                    } catch (Exception e) {
                        offlineDTO.setIdentificador(0);
                    }


                    try {
                        offlineDTO.setTipo(respuesta.getAutorizaciones().getAutorizacion().get(0).getMensajes().getMensaje().get(0).getTipo());
                    } catch (Exception e) {
                        offlineDTO.setTipo("NO DEFINIDO");
                    }

                    try {
                        offlineDTO.setInfoAdicional(respuesta.getAutorizaciones().getAutorizacion().get(0).getMensajes().getMensaje().get(0).getInformacionAdicional());
                    } catch (Exception e) {
                        offlineDTO.setInfoAdicional("NINGNUNO");
                    }
                }
            }

            if(respuesta.getNumeroComprobantes().equals("0")){
                offlineDTO.setAmbiente("PRODUCCION");
                offlineDTO.setEstado("NO AUTORIZADO SIN ENVIO RECEPCION");
                offlineDTO.setMensaje("NO DEFIIDO");
                offlineDTO.setTipo("NO DEFINIDO");
                offlineDTO.setInfoAdicional("NINGNUNO");
            }

            return offlineDTO;

        }catch (Exception e){
            return null;
        }
    }



    public void envioCorreoRecepcion(FacturaDTO factura){

        java.util.List<String> para = new ArrayList<String>();

        if (factura.getXml() != null) {
            String mail = factura.getEmailCl();
            if (mail == null || mail.equals("")) {
                mail = "german.romero@epn.edu.ec";
            }
            para.add(mail);
            String hoyFecha = "";
            DateFormat fecha = new SimpleDateFormat("yyyy/MM/dd");
            Date hoy = new Date();

            hoyFecha = fecha.format(hoy);

           // String de = "<facturacion.electronica@epn.edu.ec>";
            String subject = "EPN FACTURA ELECTRÓNICA";
            // Cuerpo mensaje

            String cabecera = "<HTML><BODY>  <p align='center' style='margin-right:50px;'> <img src='https://cem.epn.edu.ec/imagenes/logos_institucionales/big_jpg/EPN_logo_big.jpg' width= '300' height='150' style='margin-right:50px;' /> <br/> <br/>";
            String cuerpo = "<p align='center' style='margin-right:50px;'>" + "<b>Estimado(a): </b> <br/>"
                    + factura.getNombreCli() + "<br/>"
                    + "<b>Una nueva Factura Electrónica está disponible para usted  </b><br/>"
                    + "<b>REFERENCIAS </b> <br/>" + "<b>Factura Número: </b>  " + factura.getIdFactura() + " <br/>"
                    + "<b>Total Factura: </b>" + String.valueOf(factura.getTotalF()) + " USD <br/>"
                    + "<b>Fecha Factura: </b>" + hoyFecha + " <br/>" + "<b>Ambiente: </b>" + ambiente + " <br/>"
                    + "<b>Estado: </b>" + factura.getEstadoSri() + " <br/>" + "<b>Clave de Acceso: </b>"
                    + factura.getClaveacceso() + " <br/>";

            String pie = "<br/> <br/> <img src='cid:cidpie' style='margin-right:50px;'/></BODY></HTML>";

            byte[] ptext = this.remplazarCaracteresHTML(cuerpo).getBytes(ISO_8859_1);
            String cuerpoF = new String(ptext, UTF_8);

            String text = cabecera + cuerpoF + pie;

            EmailService emailService= new EmailService();
            emailService.sendEmail(para, subject, text, "", factura.getIdFactura());
        }

    }



    public String remplazarCaracteresHTML(String valor) {
        try {
            String A = "&#193;";
            String E = "&#201;";
            String I = "&#205;";
            String O = "&#211;";
            String U = "&#218;";
            String a = "&#225;";
            String e = "&#233;";
            String i = "&#237;";
            String o = "&#243;";
            String u = "&#250;";
            String Ñ = "&#209;";
            String ñ = "&#241;";

            valor = valor.replace("á", a).replace("é", e).replace("í", i).replace("ó", o).replace("ú", u)
                    .replace("Á", A).replace("É", E);
            valor = valor.replace("Í", I).replace("Ó", O).replace("Ú", U).replace("Ñ", Ñ).replace("ñ", ñ);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return valor;
    }




}
