package ec.edu.epn.consumosriautorizacion.controller;




import ec.edu.epn.consumosriautorizacion.config.SchPublicConfig;
import ec.edu.epn.consumosriautorizacion.dto.*;

import ec.edu.epn.consumosriautorizacion.service.conexionPostgres;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.context.annotation.Primary;
import org.springframework.core.env.Environment;

import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.sql.DataSource;
import java.io.Serializable;
import java.sql.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;




@Configuration
public class ConsultasFacturacionController implements Serializable {

    private static final long serialVersionUID = 1L;




      /*  @Autowired
        SchPublicConfig config;*/



    @Value("${spring.datasource.url}")
    private String url;


    @Value("${spring.datasource.username}")
    private String username;


    @Value("${spring.datasource.password}")
    private String password;


    @Value("${spring.datasource.driver-class-name}")
    private String drivers;


    public  JdbcTemplate jdbcTemplate;


   // @Primary
    @Bean//(name = "pubDataSource")
    public DataSource adminDatasource() {
        DriverManagerDataSource dataSource = new DriverManagerDataSource();
       /* dataSource.setUrl(env.getProperty("spring.datasource.url"));
        dataSource.setUsername(env.getProperty("spring.datasource.username"));
        dataSource.setPassword(env.getProperty("spring.datasource.password"));
        dataSource.setDriverClassName(env.getProperty("spring.datasource.driver-class-name"));*/

        dataSource.setUrl(url);
        dataSource.setUsername(username);
        dataSource.setPassword(password);
        dataSource.setDriverClassName(drivers);


        return dataSource;
    }

    @Bean
    public JdbcTemplate jdbcTemplate() {
        return new JdbcTemplate(adminDatasource());
    }

   /* @Autowired
    public ConsultasFacturacionController(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }*/


    public List<FacturaDTO> consumoFactContingencia(String fecha){
        List<FacturaDTO> listFact = new ArrayList<FacturaDTO>();

        try {

            listFact= jdbcTemplate().query("SELECT * FROM \"Facturacion\".factura WHERE fechafac_f like ? AND estado_sri LIKE '%CONTINGENCIA%'", BeanPropertyRowMapper.newInstance(FacturaDTO.class), "%" + fecha + "%");


            return listFact;

        }catch (Exception e){
            return null;
        }
    }


    public List<FacturaDTO> consumoFactPDF(String fecha){
        List<FacturaDTO> listFact = new ArrayList<FacturaDTO>();

        try {

            listFact= jdbcTemplate().query("SELECT * FROM \"Facturacion\".factura WHERE path IS NULL AND estado_sri = 'AUTORIZADO' AND fechafac_f like ? ", BeanPropertyRowMapper.newInstance(FacturaDTO.class), "%" + fecha + "%");


            return listFact;

        }catch (Exception e){
            return null;
        }
    }


    public FacturaDTO consumoFactura(String idFactura){

       FacturaDTO facturaDTO= new FacturaDTO();
       try {
          facturaDTO= jdbcTemplate().queryForObject("SELECT * FROM \"Facturacion\".factura WHERE trim(id_factura)=? ", BeanPropertyRowMapper.newInstance(FacturaDTO.class), idFactura);

           return facturaDTO;
       }catch (Exception e){
           return null;
       }
   }


   public List<DetallefacturaDTO> consumoDetalleFact(String idFactura){
       List<DetallefacturaDTO> listDetalleFAct= new ArrayList<DetallefacturaDTO>();
       List<DetallefacturaDTO> listDetalleFActTotal= new ArrayList<DetallefacturaDTO>();

       try {
           listDetalleFAct= jdbcTemplate().query("SELECT * FROM \"Facturacion\".detallefactura WHERE trim(id_factura)=? ", BeanPropertyRowMapper.newInstance(DetallefacturaDTO.class), idFactura);

           for(DetallefacturaDTO dto: listDetalleFAct){
               if(dto.getIdConcepto()!=null){
                   dto.setConceptoDTO(this.consumoConcepto(dto.getIdConcepto()));
               }

               if(dto.getIdServicio()!=null){
                   dto.setServicioDTO(this.consumoServicio(dto.getIdServicio()));
               }


               listDetalleFActTotal.add(dto);

           }


           return listDetalleFActTotal;

       }catch (Exception e){
           return null;
       }



   }



   public List<PagosDTO> consumoPagos(String idFactura){

       List<PagosDTO> listPagosDTO= new ArrayList<PagosDTO>();
       List<PagosDTO> listPagosDTOTotal= new ArrayList<PagosDTO>();
       try {
           listPagosDTO= jdbcTemplate().query("SELECT * FROM \"Facturacion\".pagos WHERE trim(id_factura)=? ", BeanPropertyRowMapper.newInstance(PagosDTO.class), idFactura);


           for(PagosDTO dto: listPagosDTO){

               if(dto.getIdFormaPago()!=null){

                   dto.setFormaPagoDTO(this.consumoFormaPago(dto.getIdFormaPago()));

               }

               listPagosDTOTotal.add(dto);
           }

           return listPagosDTOTotal;


       }catch (Exception e){
           return null;
       }
   }


   public FormaPagoDTO consumoFormaPago(String idFormaPago){
        FormaPagoDTO formaPagoDTO= new FormaPagoDTO();

       try {

           formaPagoDTO= jdbcTemplate().queryForObject("SELECT * FROM \"Facturacion\".forma_pago WHERE trim(id_forma_pago)=? ", BeanPropertyRowMapper.newInstance(FormaPagoDTO.class), idFormaPago);

           return formaPagoDTO;

       }catch (Exception e){
            return null;
       }
   }


   public ConceptoDTO consumoConcepto(String idConcepto){
       ConceptoDTO conceptoDTO = new ConceptoDTO();

       try {
           conceptoDTO= jdbcTemplate().queryForObject("SELECT * FROM \"Facturacion\".concepto WHERE trim(id_concepto)=? ", BeanPropertyRowMapper.newInstance(ConceptoDTO.class), idConcepto);

           return conceptoDTO;

       }catch (Exception e){
           return null;
       }
   }


   public ServicioDTO consumoServicio(String idServicio){

       ServicioDTO servicioDTO= new ServicioDTO();

       try {

           servicioDTO= jdbcTemplate().queryForObject("SELECT * FROM \"Laboratorios\".servicio WHERE trim(id_servicio)=? ", BeanPropertyRowMapper.newInstance(ServicioDTO.class), idServicio);



           return servicioDTO;
       }catch (Exception e){
           return null;
       }
   }



   public void actualizarEstadoSRIFact(FacturaDTO facturaDTO){
        try {

            String sql= "UPDATE \"Facturacion\".factura SET estado_sri = ? WHERE id_factura= ?;";
            jdbcTemplate().update(sql, facturaDTO.getEstadoFactura(), facturaDTO.getIdFactura());

        }catch (Exception e){
            e.getMessage();
        }
   }


    public void actualizarSRIFact(FacturaDTO facturaDTO){
        try {

            String sql= "UPDATE \"Facturacion\".factura SET estado_sri = ?, fechaautoriza= ? WHERE id_factura= ?;";
            jdbcTemplate().update(sql,  facturaDTO.getEstadoFactura(), facturaDTO.getFechaautoriza(), facturaDTO.getIdFactura());
            System.out.println("FACTURA  ACTUALIZADA CORRECTAMENTE");

        }catch (Exception e){
            e.getMessage();
        }
    }


    public void actualizarPathPdf(FacturaDTO facturaDTO){
        try {

            String sql= "UPDATE \"Facturacion\".factura SET path = ?, pathxml= ? WHERE id_factura= ?;";
            jdbcTemplate().update(sql,  facturaDTO.getPath(), facturaDTO.getPathxml(), facturaDTO.getIdFactura());
            System.out.println("FACTURA  ACTUALIZADA CORRECTAMENTE");

        }catch (Exception e){
            e.getMessage();
        }
    }




    public void ingresoErrorSRIFact(FacturacionOfflineDTO offlineDTO){
        try {


            long now = System.currentTimeMillis();
            Timestamp sqlTimestamp = new Timestamp(now);

            String sql= "INSERT INTO \"Facturacion\".recepcion_comprobantes_sri(" +
                        "id_documento, tipo_documento, servicio, fecha_recepcion," +
                        "estado, tipo, identificador, mensaje, info_adicional)" +
                            "VALUES (?, ?, ?, ?, ?," +
                                "?, ?, ?, ?, ?);";


            jdbcTemplate().update(sql, offlineDTO.getNumeroAutoFac(), "FACTURA", "VALIDACIÓN", sqlTimestamp,
                    offlineDTO.getEstado(), offlineDTO.getTipo(), offlineDTO.getIdentificador(), offlineDTO.getMensaje(),
                    offlineDTO.getInfoAdicional());

            System.out.println("RECEPCION INGRESADA CORRECTAMENTE");


        }catch (Exception e){
            e.getMessage();
        }
    }



    public UserFactDTO consumoUserFact(String rucCli){
        UserFactDTO userFactDTO= new UserFactDTO();

        try {

            userFactDTO= jdbcTemplate().queryForObject("SELECT * FROM \"Facturacion\".userfact WHERE trim(usuario)=? ", BeanPropertyRowMapper.newInstance(UserFactDTO.class), rucCli);

            return userFactDTO;

        }catch (Exception e){
            return null;
        }
    }


    public void actualizarUserFact(UserFactDTO userFactDTO){
        try {

            String sql= "UPDATE \"Facturacion\".userfact SET clave = ?;";
            jdbcTemplate().update(sql,  userFactDTO.getClave());
            System.out.println("USER FACT  ACTUALIZADA CORRECTAMENTE");

        }catch (Exception e){
            e.getMessage();
        }
    }



    public void ingresarUserFact(UserFactDTO userFactDTO){
        try {

            String sql= "INSERT INTO \"Facturacion\".userfact(usuario, clave) VALUES (?, ?);;";
            jdbcTemplate().update(sql, userFactDTO.getUsuario() ,userFactDTO.getClave());
            System.out.println("USER FACT  INGRESADA CORRECTAMENTE");

        }catch (Exception e){
            e.getMessage();
        }
    }






}
